#pylint: disable=too-many-public-methods, line-too-long, missing-docstring

'''Unit test for switch_java_functions'''

import io
import unittest

import switch_java_functions as funcs

SET_ALTERNATIVES = '/usr/sbin/alternatives --set '

def file_always_exists(path):
    return True

def create_stub_open_that_raises(exception):
    def stub_open(path, attribute):
        raise exception
    return stub_open

def create_stub_open_for(file_name, contents):
    def stub_open(path, attribute):
        if path != file_name:
            raise AssertionError("Excepted path: '" + file_name + "' " +
                                 "Got path: '" + path + "'")
        return io.BytesIO(contents)
    return stub_open


class SimpleUsage(unittest.TestCase):

    def __init__(self, tests):
        super(SimpleUsage, self).__init__(tests)
        self._execs = None
        self._original_exec = None

    def setUp(self):
        self._execs = []
        def modified_exec_with_test_hook(commands):
            self._execs.append(commands)
        self._original_exec = funcs._exec
        # monkey-patch _exec to avoid side-effects and record arguments
        funcs._exec = modified_exec_with_test_hook
        self._reset_globals()

    def tearDown(self):
        funcs._exec = self._original_exec
        self._reset_globals()

    def _reset_globals(self):
        funcs.JAVA = {}
        funcs.JRE = {}
        funcs.JCE = {}
        funcs.JAVAC = {}
        funcs.SDK = {}
        funcs.PLUGIN = {}
        funcs.JAVADOCDIR = {}

    def test_switch_java_changes_java_and_jre_alternatives(self):
        some_java = '1.6.0-openjdk.x86_64'
        some_java_vendor = 'openjdk'
        some_java_version = '1.6.0'
        self._populate_empty_globals(some_java)
        path_to_some_java = '/path/to/java'
        funcs.JAVA[some_java] = path_to_some_java
        path_to_some_jre = '/path/to/jre'
        funcs.JRE[some_java] = path_to_some_jre

        funcs.switch_java(some_java)

        java_alt_command = SET_ALTERNATIVES + 'java ' + path_to_some_java
        self.assertIn(java_alt_command, self._execs)
        jre_vendor_alt_command = SET_ALTERNATIVES + 'jre_' + \
                                 some_java_vendor + ' ' + path_to_some_jre
        self.assertIn(jre_vendor_alt_command, self._execs)
        jre_version_alt_command = SET_ALTERNATIVES + 'jre_' + \
                                  some_java_version + ' ' + path_to_some_jre
        self.assertIn(jre_version_alt_command, self._execs)

    def test_switch_java_changes_jce_alternatives(self):
        some_java = '1.6.0-openjdk.x86_64'
        some_java_vendor = 'openjdk'
        some_java_version = '1.6.0'
        some_java_arch = '.x86_64'
        self._populate_empty_globals(some_java)
        path_to_some_jce = '/path/to/jce'
        funcs.JCE[some_java] = path_to_some_jce

        funcs.switch_java(some_java)

        jce_alt_command = SET_ALTERNATIVES + 'jce_' + some_java_version + \
                          '_' + some_java_vendor + '_local_policy' + \
                          some_java_arch + ' ' + path_to_some_jce
        self.assertIn(jce_alt_command, self._execs)

    def test_switch_java_changes_javac_alternatives(self):
        some_java = '1.6.0-openjdk.x86_64'
        some_java_vendor = 'openjdk'
        some_java_version = '1.6.0'
        self._populate_empty_globals(some_java)
        path_to_some_javac = '/path/to/javac'
        funcs.JAVAC[some_java] = path_to_some_javac
        path_to_some_sdk = '/path/to/sdk'
        funcs.SDK[some_java] = path_to_some_sdk

        funcs.switch_java(some_java)

        javac_alt_command = SET_ALTERNATIVES + 'javac ' + path_to_some_javac
        self.assertIn(javac_alt_command, self._execs)
        javac_vendor_alt_command = SET_ALTERNATIVES + 'java_sdk_' + \
                                   some_java_vendor + ' ' + path_to_some_sdk
        self.assertIn(javac_vendor_alt_command, self._execs)
        javac_version_alt_command = SET_ALTERNATIVES + 'java_sdk_' + \
                                    some_java_version + ' ' + path_to_some_sdk
        self.assertIn(javac_version_alt_command, self._execs)

    def test_switch_java_changes_plugin_alternatives(self):
        some_java = '1.6.0-openjdk.x86_64'
        some_java_arch = '.x86_64'
        self._populate_empty_globals(some_java)
        path_to_some_plugin = '/path/to/plugin'
        funcs.PLUGIN[some_java] = path_to_some_plugin

        funcs.switch_java(some_java)

        plugin_alt_command = SET_ALTERNATIVES + 'libjavaplugin.so' + \
                             some_java_arch + ' ' + path_to_some_plugin
        self.assertIn(plugin_alt_command, self._execs)

    def test_switch_java_changes_javadocdir_alternatives(self):
        some_java = '1.6.0-openjdk.x86_64'
        self._populate_empty_globals(some_java)
        path_to_some_javadoc = '/path/to/javadoc'
        funcs.JAVADOCDIR[some_java] = path_to_some_javadoc

        funcs.switch_java(some_java)

        javadoc_alt_command = SET_ALTERNATIVES + 'javadocdir ' + \
                             path_to_some_javadoc
        self.assertIn(javadoc_alt_command, self._execs)


    def _populate_empty_globals(self, java):
        '''Initialize module-level global dictionaries with keys.'''
        funcs.JAVA[java] = ''
        funcs.JRE[java] = ''
        funcs.JCE[java] = None
        funcs.JAVAC[java] = None
        funcs.SDK[java] = None
        funcs.PLUGIN[java] = None
        funcs.JAVADOCDIR[java] = None

    @unittest.skip('implement this test')
    def test_get_java_identifiers(self):
        pass

    def test_get_java_identifiers_finds_jre_in_jdk_without_jre_subdir(self):
        alternative = '/usr/lib/jvm/java-1.8.0-openjdk/bin/java'
        def get_alternatives(name):
            return ([alternative], alternative)

        identifiers, best = funcs.get_java_identifiers(get_alternatives_func=get_alternatives)

        self.assertEqual('1.8.0-openjdk', best)
        self.assertEqual(['1.8.0-openjdk'], identifiers)

    @unittest.skip('implement this test')
    def test_get_plugin_alternatives(self):
        pass

    @unittest.skip('implement this test')
    def test_get_javadocdir_alternatives(self):
        pass

    def test_get_alternatives_raises_JavaOpenError_on_missing_alternative_file(self):
        mock_open = create_stub_open_that_raises(IOError())

        with self.assertRaises(funcs.JavaOpenError):
            funcs.get_alternatives('foo', mock_open, None)

    def test_get_alternatives_raises_JavaParseError_on_empty_file(self):
        some_master = 'foo'
        expected_alternative = '/var/lib/alternatives/' + some_master
        contents = ''
        mock_open = create_stub_open_for(expected_alternative, contents)

        with self.assertRaises(funcs.JavaParseError):
            funcs.get_alternatives(some_master, mock_open, file_always_exists)

    def test_get_alternatives_empty_alternatives_best_is_negative(self):
        some_master = 'foo'
        expected_alternative = '/var/lib/alternatives/' + some_master
        contents = 'auto\n/usr/bin/java\n\n'
        mock_open = create_stub_open_for(expected_alternative, contents)

        alternatives, best = funcs.get_alternatives(some_master,
                                                    mock_open,
                                                    file_always_exists)

        self.assertListEqual([], alternatives)
        self.assertEquals(-1, best)

    def test_get_alternatives_parses_alternatives(self):
        some_master = 'foo'
        expected_alternative = '/var/lib/alternatives/' + some_master
        first_alt = '/some/java'
        first_priority = 80
        contents = 'auto\n' + \
                   '/usr/bin/java\n' + \
                   '\n' + \
                   first_alt + '\n' + \
                   str(first_priority) + '\n'
        mock_open = create_stub_open_for(expected_alternative, contents)

        alternatives, best = funcs.get_alternatives(some_master,
                                                    mock_open,
                                                    file_always_exists)

        self.assertListEqual([first_alt], alternatives)
        self.assertEquals(0, best)

    def test_get_alternatives_selects_best_alternatives(self):
        some_master = 'foo'
        expected_alternative = '/var/lib/alternatives/' + some_master
        first_alt = '/some/java'
        first_priority = 80
        second_alt = '/some/thing/else'
        second_priority = 1000
        third_alt = '/some/other/else'
        third_priority = 100
        in_alternatives = [first_alt, str(first_priority),
                           second_alt, str(second_priority),
                           third_alt, str(third_priority)]
        highest_priority_index = 1
        contents = 'auto\n' + \
                   '/usr/bin/java\n' + \
                   '\n' + \
                   '\n'.join(in_alternatives) + \
                   '\n'
        mock_open = create_stub_open_for(expected_alternative, contents)

        alternatives, best = funcs.get_alternatives(some_master,
                                                    mock_open,
                                                    file_always_exists)

        self.assertListEqual([first_alt, second_alt, third_alt],
                             alternatives)
        self.assertEquals(highest_priority_index, best)

    @unittest.skip('implement this test')
    def test_initialize_alternative_dictionaries(self):
        pass

    def test_initialize_alternative_dictionaries_picks_up_java_with_jre_subdir(self):
        identifiers = [('1.8.0-openjdk', True)] # True => JDK
        self.assertEqual({}, funcs.JAVA)

        def path_exists(path):
            return True

        funcs.initialize_alternatives_dictionaries(identifiers, path_exists=path_exists)

        self.assertEqual({'1.8.0-openjdk': '/usr/lib/jvm/java-1.8.0-openjdk/jre/bin/java'},
                         funcs.JAVA)

    def test_initialize_alternative_dictionaries_picks_up_java_without_jre_subdir(self):
        identifiers = [('1.8.0-openjdk', True)] # True => JDK
        self.assertEqual({}, funcs.JAVA)

        def path_exists(path):
            return not '/jre' in path

        funcs.initialize_alternatives_dictionaries(identifiers, path_exists=path_exists)

        self.assertEqual({'1.8.0-openjdk': '/usr/lib/jvm/java-1.8.0-openjdk/bin/java'},
                         funcs.JAVA)

    def test_initialize_alternative_dictionaries_picks_up_jre_without_jre_subdir(self):
        identifiers = [('1.8.0-openjdk', True)] # True => JDK
        self.assertEqual({}, funcs.JRE)

        def path_exists(path):
            return not '/jre' in path

        funcs.initialize_alternatives_dictionaries(identifiers, path_exists=path_exists)

        self.assertEqual({'1.8.0-openjdk': '/usr/lib/jvm/java-1.8.0-openjdk'},
                         funcs.JRE)

    def test_get_default_java_command(self):
        result = funcs.get_default_java_command('/foo/bar/baz/spam/eggs')
        self.assertEquals(result, None)

    def test_get_pretty_names(self):
        result = funcs.get_pretty_names([])
        self.assertEquals(result, {})

    def test_get_pretty_names_x86_64(self):
        # this test is not exhaustive. specific tests on all names are
        # done separately via test_get_pretty_name_* tests
        java1 = '1.7.0-openjdk'
        java2 = '1.5.0-sun.x86_64'
        result = funcs.get_pretty_names([java1, java2])
        self.assertEquals(result, {java1 : 'OpenJDK 1.7.0', java2: 'Sun 1.5.0 (x86_64)'})

    def test_get_pretty_names_i686(self):
        # this test is not exhaustive. specific tests on all names are
        # done separately via test_get_pretty_name_* tests
        java1 = '1.7.0-openjdk.i686'
        java2 = '1.5.0-sun.i386'
        result = funcs.get_pretty_names([java1, java2])
        self.assertEquals(result, {java1 : 'OpenJDK 1.7.0 (i686)', java2: 'Sun 1.5.0 (i386)'})

    def test_get_pretty_name_no_arch(self):
        java_name = '1.7.0-openjdk'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'OpenJDK 1.7.0')

    def test_get_pretty_name_i686(self):
        java_name = '1.7.0-openjdk.i686'
        result = funcs.get_pretty_name(java_name)
        # Yes, i686 -> 64-bit.
        self.assertEquals(result, 'OpenJDK 1.7.0 (i686)')

    def test_get_pretty_name_x86_64(self):
        java_name = '1.7.0-openjdk.x86_64'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'OpenJDK 1.7.0 (x86_64)')

    def test_get_pretty_name_vendor_is_icedtea(self):
        java_name = '1.7.0-icedtea'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'IcedTea 1.7.0')

    def test_get_pretty_name_vendor_is_sun(self):
        java_name = '1.7.0-sun'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'Sun 1.7.0')

    def test_get_pretty_name_vendor_is_oracle(self):
        java_name = '1.7.0-oracle'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'Oracle 1.7.0')

    def test_get_pretty_name_vendor_is_ibm(self):
        java_name = '1.7.0-ibm'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'IBM 1.7.0')

    def test_get_java_split_with_arch(self):
        java = '1.7.0-openjdk.x86_64'
        result = funcs.get_java_split(java)
        self.assertEqual(result, ('openjdk', '1.7.0', '.x86_64'))

    def test_get_java_split_without_arch(self):
        java = '1.6.0-gcj'
        result = funcs.get_java_split(java)
        self.assertEqual(result, ('gcj', '1.6.0', ''))

    def test_get_pretty_name_openjdk7_full_nvr(self):
        java_name = '1.7.0-openjdk-1.7.0.51-2.4.5.5.el7.x86_64'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'OpenJDK 1.7.0 (1.7.0.51-2.4.5.5.el7.x86_64)')

    def test_get_pretty_name_openjdk7_full_nvr_fedora(self):
        java_name = '1.7.0-openjdk-1.7.0.51-2.4.5.5.fc21.x86_64'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'OpenJDK 1.7.0 (1.7.0.51-2.4.5.5.fc21.x86_64)')

    def test_get_pretty_name_openjdk6_partial_nvr(self):
        java_name = '1.6.0-openjdk-1.6.0.0.x86_64'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'OpenJDK 1.6.0 (1.6.0.0.x86_64)')

    def test_get_pretty_name_sun6_jsut_arch(self):
        java_name = '1.6.0-sun.x86_64'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'Sun 1.6.0 (x86_64)')

    def test_get_pretty_name_sun6_partial_nvr(self):
        java_name = '1.6.0-sun-1.6.0.45.x86_64'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'Sun 1.6.0 (1.6.0.45.x86_64)')

    def test_get_pretty_name_openjdk6_just_arch(self):
        java_name = '1.6.0-openjdk.x86_64'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'OpenJDK 1.6.0 (x86_64)')

    def test_get_pretty_name_oracle7_full_nvr(self):
        java_name = '1.7.0-oracle-1.7.0.51-1jpp.1.el7.i386'
        result = funcs.get_pretty_name(java_name)
        self.assertEquals(result, 'Oracle 1.7.0 (1.7.0.51-1jpp.1.el7.i386)')

    # this is testing above items
    # adding duplicated or similar entries
    def test_list_of_javas(self):
        javas = ['1.7.0-openjdk-1.7.0.51-2.4.5.5.el7.x86_64',
                 '1.7.0-openjdk-1.7.0.55-2.4.7.0.el7.x86_64',
                 '1.6.0-openjdk-1.6.0.0.x86_64',
                 '1.7.0-oracle',
                 '1.6.0-openjdk.x86_64',
                 '1.7.0-oracle-1.7.0.51-1jpp.1.el7.i386',
                 '1.6.0-sun-1.6.0.45.x86_64',
                 '1.7.0-oracle-1.7.0.51-1jpp.1.el7.x86_64',
                 '1.6.0-sun.x86_64',
                 '1.7.1-ibm-1.7.1.0.0-1jpp.4.el7.x86_64',
                 '1.6.0-sun.x86_64',
                 '1.7.0-oracle-1.7.0.51-1jpp.1.el7.x86_64',
                 '1.7.0-ibm',
                 '1.7.0-openjdk',
                 '1.7.0-openjdk-1.7.0.51-2.4.5.5.el7.i386',
                 '1.7.0-openjdk-1.7.0.51-2.4.5.5.fc21.i386']
        result = funcs.get_pretty_names(javas)
        self.assertEquals(result[javas[0]], 'OpenJDK 1.7.0 (1.7.0.51-2.4.5.5.el7.x86_64)')
        self.assertEquals(result[javas[1]], 'OpenJDK 1.7.0 (1.7.0.55-2.4.7.0.el7.x86_64)')
        self.assertEquals(result[javas[2]], 'OpenJDK 1.6.0 (1.6.0.0.x86_64)')
        self.assertEquals(result[javas[3]], 'Oracle 1.7.0')
        self.assertEquals(result[javas[4]], 'OpenJDK 1.6.0 (x86_64)')
        self.assertEquals(result[javas[5]], 'Oracle 1.7.0 (1.7.0.51-1jpp.1.el7.i386)')
        self.assertEquals(result[javas[6]], 'Sun 1.6.0 (1.6.0.45.x86_64)')
        self.assertEquals(result[javas[7]], 'Oracle 1.7.0 (1.7.0.51-1jpp.1.el7.x86_64)')
        self.assertEquals(result[javas[8]], 'Sun 1.6.0 (x86_64)')
        self.assertEquals(result[javas[9]], 'IBM 1.7.1 (1.7.1.0.0-1jpp.4.el7.x86_64)')
        self.assertEquals(result[javas[10]], 'Sun 1.6.0 (x86_64)');
        self.assertEquals(result[javas[11]], 'Oracle 1.7.0 (1.7.0.51-1jpp.1.el7.x86_64)')
        self.assertEquals(result[javas[12]], 'IBM 1.7.0')
        self.assertEquals(result[javas[13]], 'OpenJDK 1.7.0')
        self.assertEquals(result[javas[14]], 'OpenJDK 1.7.0 (1.7.0.51-2.4.5.5.el7.i386)')
        self.assertEquals(result[javas[15]], 'OpenJDK 1.7.0 (1.7.0.51-2.4.5.5.fc21.i386)')

    def test_version_sort_split(self):
        a = funcs.version_sort_split("1.7.0-openjdk-1.7.0.51-2.4.5.5.el7.i386")
        self.assertEquals(a[0], '1')
        self.assertEquals(a[1], '7')
        self.assertEquals(a[2], '0')
        self.assertEquals(a[3], 'openjdk')
        self.assertEquals(a[4], '1')
        self.assertEquals(a[5], '7')
        self.assertEquals(a[6], '0')
        self.assertEquals(a[7], 'openjdk')
        self.assertEquals(a[8], '1')
        self.assertEquals(a[9], '7')
        self.assertEquals(a[10], '0')
        self.assertEquals(a[11], '51')
        self.assertEquals(a[12], '2')
        self.assertEquals(a[13], '4')
        self.assertEquals(a[14], '5')
        self.assertEquals(a[15], '5')
        self.assertEquals(a[16], 'el7')
        self.assertEquals(a[17], 'i386')


    def test_version_sort_split_on_short(self):
        a = funcs.version_sort_split("1.7.0-openjdk")
        self.assertEquals(a[0], '1')
        self.assertEquals(a[1], '7')
        self.assertEquals(a[2], '0')
        self.assertEquals(a[3], 'openjdk')
        self.assertEquals(a[4], '1')
        self.assertEquals(a[5], '7')
        self.assertEquals(a[6], '0')
        self.assertEquals(a[7], 'openjdk')

    def test_version_sort_split_on_short_with_arch(self):
        a = funcs.version_sort_split("1.6.0-openjdk.x686")
        self.assertEquals(a[0], '1')
        self.assertEquals(a[1], '6')
        self.assertEquals(a[2], '0')
        self.assertEquals(a[3], 'openjdk')
        self.assertEquals(a[4], '1')
        self.assertEquals(a[5], '6')
        self.assertEquals(a[6], '0')
        self.assertEquals(a[7], 'openjdk')
        self.assertEquals(a[8], 'x686')

    def test_version_sort(self):
        a=funcs.version_sort("1.7.0-openjdk-1.el7.x86_64", "1.7.0-openjdk-1.el7.x86_64")
        self.assertEquals(a, 0)
        a=funcs.version_sort("1.7.0-openjdk-01.el7.x86_64", "1.7.0-openjdk-1.el7.x86_64")
        self.assertEquals(a, 0)
        a=funcs.version_sort("1.7.0-openjdk-1.el7.x86_64", "1.007.0-openjdk-00001.el7.x86_64")
        self.assertEquals(a, 0)
        a=funcs.version_sort("1.7.0-openjdk-2.el7.x86_64", "1.7.0-openjdk-1.el7.x86_64")
        self.assertEquals(a, -1)
        a=funcs.version_sort("1.7.0-openjdk-2.el7.x86_64", "1.7.0-openjdk-3.el7.x86_64")
        self.assertEquals(a, 1)
        a=funcs.version_sort("1.7.0-a-0.el7.x86_64", "1.7.0-b-0.el7.x86_64")
        self.assertEquals(a, 1)
        a=funcs.version_sort("1.7.0-az-0.el7.x86_64", "1.7.0-ax-0.el7.x86_64")
        self.assertEquals(a, -1)
        a=funcs.version_sort("1.7.0-openjdk-1.7.0.51-2.4.5.5.fc21.i386", "1.7.0-openjdk")
        self.assertEquals(a, 0) # see comment in version_sort
        a=funcs.version_sort("1.7.0-oracle", "1.7.0-openjdk.x86_64")
        self.assertEquals(a, -1)








